using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;

namespace RegularLanguages
{
    public partial class DFAPreviewForm : Form
    {
        public DFAPreviewForm(DFA<char> dfa)
        {
            InitializeComponent();
            if (dfa == null)
            {
                this.decideButton.Enabled = false;
            }
            else
            {
                this.dfa = dfa;
                this.dfaTextBox.Text = dfa.ToString();
                this.commentTextBox.Text = dfa.Comment;
                this.decideButton.Enabled = true;
            }
        }

        private DFA<char> dfa;

        public void Decide()
        {
            if (this.dfa == null) return;
            string text = this.decideTextBox.Text;
            if (!this.dfa.Alphabet.Contains(text))
            {
                MessageBox.Show("All the letters of the word must be from the alphabet.", "Warning");
                return;
            }
            if (this.dfa.Decide(text))
                MessageBox.Show("Accept", "Information");
            else
                MessageBox.Show("Reject", "Information");
        }

        private void decideButton_Click(object sender, EventArgs e)
        {
            this.Decide();
            this.decideTextBox.Clear();
        }

        private void decideTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.Decide();
        }

        private void decideTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.decideTextBox.Clear();
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            if (this.dfa == null) return;
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            DialogResult saveFileDialogResult = saveFileDialog.ShowDialog();
            if (saveFileDialogResult == DialogResult.OK)
            {
                XmlSerializer dfaSerializer =
                    new XmlSerializer(typeof(DFAXML<char>));
                FileStream fileSaveStream =
                    new FileStream(saveFileDialog.FileName, FileMode.Create);
                DFAXML<char> dfaXML = new DFAXML<char>(this.dfa);
                dfaSerializer.Serialize(fileSaveStream, dfaXML);
                fileSaveStream.Close();
            }
        }

        private void copyButton_Click(object sender, EventArgs e)
        {
            if (this.dfa == null) return;
            XmlSerializer dfaSerializer =
                new XmlSerializer(typeof(DFAXML<char>));
            StringWriter stringWriter = new StringWriter();
            DFAXML<char> dfaXML = new DFAXML<char>(this.dfa);
            dfaSerializer.Serialize(stringWriter, dfaXML);
            Clipboard.SetText(stringWriter.ToString());
            stringWriter.Close();
        }

        private void viewButton_Click(object sender, EventArgs e)
        {
            if (this.dfa == null) return;
            XmlSerializer dfaSerializer =
                new XmlSerializer(typeof(DFAXML<char>));
            StringWriter stringWriter = new StringWriter();
            DFAXML<char> dfaXML = new DFAXML<char>(this.dfa);
            dfaSerializer.Serialize(stringWriter, dfaXML);
            XMLPreviewForm xmlPreviewForm = new XMLPreviewForm(stringWriter.ToString());
            stringWriter.Close();
            xmlPreviewForm.ShowDialog();
        }
    }
}