using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace RegularLanguages
{
    public partial class RegexForm : Form, LanguageForm<char>
    {
        public RegexForm()
        {
            InitializeComponent();
            this.lockAlphabet = false;
            this.alphabet = null;
            this.UpdateUI();
        }

        public RegexForm(Alphabet<char> alphabet)
        {
            InitializeComponent();
            this.lockAlphabet = true;
            this.alphabet = alphabet;
            this.UpdateUI();
        }

        public RegexForm(Alphabet<char> alphabet, string regexPattern)
        {
            InitializeComponent();
            this.lockAlphabet = true;
            this.alphabet = alphabet;
            this.regexPatternTextBox.Text = regexPattern;
            this.UpdateUI();
        }

        public RegexForm(LanguageRegex<char> languageRegex)
        {
            InitializeComponent();
            this.lockAlphabet = true;
            this.alphabet = languageRegex.Alphabet;
            this.regexPatternTextBox.Text = languageRegex.RegexPattern;
            this.UpdateUI();
        }

        public ILanguage<char> Language
        {
            get
            {
                try
                {
                    LanguageRegex<char> language =
                        new LanguageRegex<char>(this.alphabet, this.regexPatternTextBox.Text);
                    return language;
                }
                catch
                {
                    return null;
                }
            }
        }

        private void UpdateUI()
        {
            if (this.lockAlphabet)
                this.alphabetButton.Enabled = false;
            else
            {
                this.alphabetButton.Enabled = true;
                this.okButton.Enabled = false;
                this.cancelButton.Enabled = false;
            }
            if (this.alphabet == null)
            {
                this.regexPatternTextBox.Enabled = false;
                this.validatePatternButton.Enabled = false;
                this.helpButton.Enabled = false;
                this.regexTestTextBox.Enabled = false;
                this.decideButton.Enabled = false;
            }
            else
            {
                this.regexPatternTextBox.Enabled = true;
                this.validatePatternButton.Enabled = true;
                this.helpButton.Enabled = true;
                this.regexTestTextBox.Enabled = true;
                this.decideButton.Enabled = true;
            }
        }

        private Regex RegexTest()
        {
            try
            {
                Regex regex = new Regex(this.regexPatternTextBox.Text);
                return regex;
            }
            catch
            {
                MessageBox.Show("Invalid regex pattern.", "Warning");
                return null;
            }
        }

        private void RegexIsMatch(string word)
        {
            if (!this.alphabet.Contains(word))
            {
                MessageBox.Show("All the letters of the word must be from the alphabet.", "Warning");
                return;
            }
            Regex regex = this.RegexTest();
            if (regex != null)
            {
                if (regex.IsMatch(word))
                    MessageBox.Show("Accept", "Information");
                else MessageBox.Show("Reject", "Information");
            }
        }

        private bool lockAlphabet;
        private Alphabet<char> alphabet;
        private bool canClose = false;

        private void alphabetButton_Click(object sender, EventArgs e)
        {
            AlphabetForm alphabetForm = new AlphabetForm();
            DialogResult dialogResult = alphabetForm.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                if (alphabetForm.Alphabet == "")
                {
                    this.alphabet = null;
                }
                else
                {
                    this.alphabet = new Alphabet<char>(alphabetForm.Alphabet);
                }
                this.UpdateUI();
            }
        }

        private void regexTestTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                this.RegexIsMatch(this.regexTestTextBox.Text);
                this.regexTestTextBox.Clear();
            }
        }

        private void regexTestTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.regexTestTextBox.Clear();
        }

        private void validatePatternButton_Click(object sender, EventArgs e)
        {
            if (this.RegexTest() != null)
                MessageBox.Show("Regex pattern is valid.", "Information");
        }

        private void helpButton_Click(object sender, EventArgs e)
        {
            RegexHelpForm regexSamplesForm = new RegexHelpForm();
            regexSamplesForm.ShowDialog();
        }

        private void decideButton_Click(object sender, EventArgs e)
        {
            this.RegexIsMatch(this.regexTestTextBox.Text);
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.canClose = (this.RegexTest() != null);
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.canClose = true;
        }

        private void RegexForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.None)
                e.Cancel = !this.canClose;
            else e.Cancel = false;
        }
    }
}