using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;
using RegularLanguages;

namespace RestartingAutomaton
{
    public partial class RestartingAutomatonForm : Form
    {
        public RestartingAutomatonForm()
        {
            InitializeComponent();
            this.alphabet = null;
            this.UpdateUI();
        }

        private void ClearUI()
        {
            this.acceptingMetainstructionListView.Items.Clear();
            this.reducingMetainstructionListView.Items.Clear();
        }

        private void UpdateUI()
        {
            if (this.alphabet == null)
            {
                this.newToolStripMenuItem.Enabled = true;
                this.openToolStripMenuItem.Enabled = true;
                this.saveToolStripMenuItem.Enabled = false;
                this.clearToolStripMenuItem.Enabled = false;
                this.wordToWordToolStripMenuItem.Enabled = false;
                this.wordToAllToolStripMenuItem.Enabled = false;
                this.startServerToolStripMenuItem.Enabled = false;
                this.acceptingMetainstructionListView.Items.Clear();
                this.reducingMetainstructionListView.Items.Clear();
                this.acceptingMetainstructionListView.Enabled = false;
                this.reducingMetainstructionListView.Enabled = false;
                this.addAcceptingMeatinstructionButton.Enabled = false;
                this.addReducingMetainstructionButton.Enabled = false;
                this.acceptingNameTextBox.Enabled = false;
                this.reducingNameTextBox.Enabled = false;
                this.acceptAllCheckButton.Enabled = false;
                this.acceptNullCheckButton.Enabled = false;
                this.acceptInvCheckButton.Enabled = false;
                this.reduceAllCheckButton.Enabled = false;
                this.reduceNullCheckButton.Enabled = false;
                this.reduceInvCheckButton.Enabled = false;
            }
            else
            {
                this.newToolStripMenuItem.Enabled = true;
                this.openToolStripMenuItem.Enabled = true;
                this.saveToolStripMenuItem.Enabled = true;
                this.clearToolStripMenuItem.Enabled = true;
                this.wordToWordToolStripMenuItem.Enabled = true;
                this.wordToAllToolStripMenuItem.Enabled = true;
                this.startServerToolStripMenuItem.Enabled = true;
                this.acceptingMetainstructionListView.Enabled = true;
                this.reducingMetainstructionListView.Enabled = true;
                this.addAcceptingMeatinstructionButton.Enabled = true;
                this.addReducingMetainstructionButton.Enabled = true;
                this.acceptingNameTextBox.Enabled = true;
                this.reducingNameTextBox.Enabled = true;
                this.acceptAllCheckButton.Enabled = true;
                this.acceptNullCheckButton.Enabled = true;
                this.acceptInvCheckButton.Enabled = true;
                this.reduceAllCheckButton.Enabled = true;
                this.reduceNullCheckButton.Enabled = true;
                this.reduceInvCheckButton.Enabled = true;
            }
            this.UpdateAcceptingMetainstructionUI();
            this.UpdateReducingMetainstructionUI();
        }

        private void UpdateAcceptingMetainstructionUI()
        {
            if (this.acceptingMetainstructionListView.SelectedItems.Count == 1)
            {
                this.deleteAcceptingMetainstructionButton.Enabled = true;
                this.wizardAcceptingLanguageButton.Enabled = true;
                this.editAcceptingLanguageButton.Enabled =
                    this.acceptingMetainstructionListView.SelectedItems[0].SubItems[1].Tag != null;
                this.acceptingNameTextBox.Text =
                    this.acceptingMetainstructionListView.SelectedItems[0].SubItems[0].Text;
            }
            else
            {
                this.deleteAcceptingMetainstructionButton.Enabled = false;
                this.wizardAcceptingLanguageButton.Enabled = false;
                this.editAcceptingLanguageButton.Enabled = false;
                this.acceptingNameTextBox.Text = "";
            }
        }

        private void UpdateReducingMetainstructionUI()
        {
            if (this.reducingMetainstructionListView.SelectedItems.Count == 1)
            {
                this.deleteReducingMetainstructionButton.Enabled = true;
                this.wizardLeftLanguageButton.Enabled = true;
                this.editLeftLanguageButton.Enabled =
                    this.reducingMetainstructionListView.SelectedItems[0].SubItems[1].Tag != null;
                this.wizardRightLanguageButton.Enabled = true;
                this.editRightLanguageButton.Enabled =
                    this.reducingMetainstructionListView.SelectedItems[0].SubItems[4].Tag != null;
                this.fromWordTextBox.Enabled = true;
                this.toWordTextBox.Enabled = true;
                this.reducingNameTextBox.Text =
                    this.reducingMetainstructionListView.SelectedItems[0].SubItems[0].Text;
                this.fromWordTextBox.Text =
                    this.reducingMetainstructionListView.SelectedItems[0].SubItems[2].Text;
                this.toWordTextBox.Text =
                    this.reducingMetainstructionListView.SelectedItems[0].SubItems[3].Text;
            }
            else
            {
                this.deleteReducingMetainstructionButton.Enabled = false;
                this.wizardLeftLanguageButton.Enabled = false;
                this.editLeftLanguageButton.Enabled = false;
                this.wizardRightLanguageButton.Enabled = false;
                this.editRightLanguageButton.Enabled = false;
                this.fromWordTextBox.Enabled = false;
                this.toWordTextBox.Enabled = false;
                this.reducingNameTextBox.Text = "";
                this.fromWordTextBox.Text = "";
                this.toWordTextBox.Text = "";
            }
        }

        private void ListViewCheckAll(ListView listView)
        {
            foreach (ListViewItem listViewItem in listView.Items)
                listViewItem.Checked = true;
        }

        private void ListViewCheckNull(ListView listView)
        {
            foreach (ListViewItem listViewItem in listView.Items)
                listViewItem.Checked = false;
        }

        private void ListViewCheckInv(ListView listView)
        {
            foreach (ListViewItem listViewItem in listView.Items)
                listViewItem.Checked = !listViewItem.Checked;
        }

        private bool ListViewContainsName(ListView listView, string name, int columnIndex)
        {
            foreach (ListViewItem listViewItem in listView.Items)
                if (listViewItem.SubItems[columnIndex].Text == name)
                    return true;
            return false;
        }

        private string ListViewNewName(ListView listView, string prefix, int columnIndex)
        {
            int index = 0;
            while (this.ListViewContainsName(listView, prefix + index.ToString("D2"), columnIndex))
                ++index;
            return prefix + index.ToString("D2");
        }

        private string ListViewAddName(ListView listView, string prefix, string name, int columnIndex)
        {
            string trimName = name.Trim();
            if (trimName == "" || this.ListViewContainsName(listView, trimName, columnIndex))
                return this.ListViewNewName(listView, prefix, columnIndex);
            else return trimName;
        }

        private void DataToUI()
        {
            this.acceptingMetainstructionListView.Items.Clear();
            this.reducingMetainstructionListView.Items.Clear();
            if (this.restartingAutomaton != null)
            {
                foreach (AcceptingMetainstruction<char> acceptingMetainstruction in
                    this.restartingAutomaton.AcceptingMetainstructions)
                {
                    ListViewItem acceptingMetainstructionItem =
                        acceptingMetainstructionListView.Items.Add(
                            "Accepting Metainstruction");
                    acceptingMetainstructionItem.Checked = true;
                    acceptingMetainstructionItem.SubItems.Add("");
                    acceptingMetainstructionItem.SubItems[0].Text =
                        acceptingMetainstruction.Name;
                    if (acceptingMetainstruction.AcceptingLanguage == null)
                        acceptingMetainstructionItem.SubItems[1].Text = "null";
                    else
                        acceptingMetainstructionItem.SubItems[1].Text =
                            acceptingMetainstruction.AcceptingLanguage.ToString();
                    acceptingMetainstructionItem.SubItems[1].Tag =
                        acceptingMetainstruction.AcceptingLanguage;
                }
                foreach (ReducingMetainstruction<char> reducingMetainstruction in
                    this.restartingAutomaton.ReducingMetainstructions)
                {
                    ListViewItem reducingMetainstructionItem =
                        reducingMetainstructionListView.Items.Add(
                            "Reducing Metainstruction");
                    reducingMetainstructionItem.Checked = true;
                    reducingMetainstructionItem.SubItems.Add("");
                    reducingMetainstructionItem.SubItems.Add("");
                    reducingMetainstructionItem.SubItems.Add("");
                    reducingMetainstructionItem.SubItems.Add("");
                    reducingMetainstructionItem.SubItems[0].Text =
                        reducingMetainstruction.Name;
                    if (reducingMetainstruction.LeftLanguage == null)
                        reducingMetainstructionItem.SubItems[1].Text = "null";
                    else
                        reducingMetainstructionItem.SubItems[1].Text =
                            reducingMetainstruction.LeftLanguage.ToString();
                    reducingMetainstructionItem.SubItems[1].Tag =
                        reducingMetainstruction.LeftLanguage;
                    reducingMetainstructionItem.SubItems[2].Text =
                        reducingMetainstruction.FromWord.ToString();
                    reducingMetainstructionItem.SubItems[3].Text =
                        reducingMetainstruction.ToWord.ToString();
                    if (reducingMetainstruction.RightLanguage == null)
                        reducingMetainstructionItem.SubItems[4].Text = "null";
                    else
                        reducingMetainstructionItem.SubItems[4].Text =
                            reducingMetainstruction.RightLanguage.ToString();
                    reducingMetainstructionItem.SubItems[4].Tag =
                        reducingMetainstruction.RightLanguage;
                }
            }
        }

        private bool UIToData(bool onlyChecked, bool onlyConsistent, bool showMessages)
        {
            if (this.alphabet != null)
            {
                this.restartingAutomaton = new RestartingAutomaton<char>(this.alphabet);
                foreach (ListViewItem acceptingMetainstructionItem in
                    this.acceptingMetainstructionListView.Items)
                    if (!onlyChecked || acceptingMetainstructionItem.Checked)
                    {
                        if (!onlyConsistent || acceptingMetainstructionItem.SubItems[1].Tag != null)
                        {
                            AcceptingMetainstruction<char> acceptingMetainstruction =
                                new AcceptingMetainstruction<char>(this.alphabet);
                            acceptingMetainstruction.Name =
                                acceptingMetainstructionItem.SubItems[0].Text;
                            if (acceptingMetainstructionItem.SubItems[1].Tag != null)
                                acceptingMetainstruction.AcceptingLanguage =
                                    acceptingMetainstructionItem.SubItems[1].Tag as ILanguage<char>;
                            this.restartingAutomaton.AddAcceptingMetainstruction(
                                acceptingMetainstruction);
                        }
                        else
                        {
                            if (showMessages)
                                MessageBox.Show("Accepting metainstruction " +
                                    acceptingMetainstructionItem.SubItems[0].Text +
                                    " is not defined.", "Information");
                            return false;
                        }
                    }
                foreach (ListViewItem reducingMetainstructionItem in
                    this.reducingMetainstructionListView.Items)
                    if (!onlyChecked || reducingMetainstructionItem.Checked)
                    {
                        if (!onlyConsistent ||
                            (reducingMetainstructionItem.SubItems[1].Tag != null &&
                             reducingMetainstructionItem.SubItems[4].Tag != null))
                        {
                            ReducingMetainstruction<char> reducingMetainstruction =
                                new ReducingMetainstruction<char>(this.alphabet);
                            reducingMetainstruction.Name =
                                reducingMetainstructionItem.SubItems[0].Text;
                            if (reducingMetainstructionItem.SubItems[1].Tag != null)
                                reducingMetainstruction.LeftLanguage =
                                    reducingMetainstructionItem.SubItems[1].Tag as ILanguage<char>;
                            if (reducingMetainstructionItem.SubItems[4].Tag != null)
                                reducingMetainstruction.RightLanguage =
                                    reducingMetainstructionItem.SubItems[4].Tag as ILanguage<char>;
                            reducingMetainstruction.FromWord = new GenericString<char>(
                                reducingMetainstructionItem.SubItems[2].Text);
                            reducingMetainstruction.ToWord = new GenericString<char>(
                                reducingMetainstructionItem.SubItems[3].Text);
                            if (onlyConsistent &&
                                reducingMetainstruction.FromWord.Length <=
                                    reducingMetainstruction.ToWord.Length)
                            {
                                if (showMessages)
                                    MessageBox.Show("In reducing metainstruction " +
                                        reducingMetainstructionItem.SubItems[0].Text +
                                        " you cannot reduce " +
                                        reducingMetainstructionItem.SubItems[2].Text +
                                        " to " +
                                        reducingMetainstructionItem.SubItems[3].Text +
                                        ".", "Information");
                                return false;
                            }
                            this.restartingAutomaton.AddReducingMetainstruction(
                                reducingMetainstruction);
                        }
                        else
                        {
                            if (showMessages)
                                MessageBox.Show("Reducing metainstruction " +
                                    reducingMetainstructionItem.SubItems[0].Text +
                                    " is not defined.", "Information");
                            return false;
                        }
                    }                    
                return true;
            }
            else return false;
        }

        private Alphabet<char> alphabet;
        private RestartingAutomaton<char> restartingAutomaton;

        private bool LanguageWizard(ref ILanguage<char> language)
        {
            LanguageWizardForm languageWizardForm =
                new LanguageWizardForm(this.alphabet);
            DialogResult dialogResult = languageWizardForm.ShowDialog();
            language = languageWizardForm.Language;
            return dialogResult == DialogResult.OK;
        }

        private void LanguageWizard(ListViewItem.ListViewSubItem languageListViewSubItem)
        {
            ILanguage<char> language = null;
            if (LanguageWizard(ref language))
            {
                languageListViewSubItem.Tag = language;
                if (language != null)
                    languageListViewSubItem.Text = language.ToString();
                else languageListViewSubItem.Text = "null";
                this.UpdateUI();
            }
        }

        private void LanguageEdit(ListViewItem.ListViewSubItem languageListViewSubItem)
        {
            if (languageListViewSubItem.Tag != null)
            {
                Form languageForm =
                    LanguageFormFactory.GetLanguageFormInstance(
                        languageListViewSubItem.Tag as ILanguageData<char>);
                DialogResult dialogResult = languageForm.ShowDialog();
                if (dialogResult == DialogResult.OK)
                {
                    ILanguage<char> language = (languageForm as LanguageForm<char>).Language;
                    languageListViewSubItem.Tag = language;
                    if (language != null)
                        languageListViewSubItem.Text = language.ToString();
                    else
                        languageListViewSubItem.Text = "null";
                }
            }
        }

        private void addAcceptingMetainstructionButton_Click(object sender, EventArgs e)
        {
            ListViewItem acceptingMetainstructionItem =
                this.acceptingMetainstructionListView.Items.Add("Accepting Metainstruction");
            acceptingMetainstructionItem.Checked = true;
            acceptingMetainstructionItem.SubItems.Add("");
            acceptingMetainstructionItem.SubItems[0].Text =
                this.ListViewAddName(this.acceptingMetainstructionListView,
                    "A", this.acceptingNameTextBox.Text, 0);
            acceptingMetainstructionItem.SubItems[1].Text = "null";
            acceptingMetainstructionItem.SubItems[1].Tag = null;
        }

        private void deleteAcceptingMetainstructionButton_Click(object sender, EventArgs e)
        {
            this.acceptingMetainstructionListView.Items.Remove(
                this.acceptingMetainstructionListView.SelectedItems[0]);
        }

        private void wizardAcceptingLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageWizard(this.acceptingMetainstructionListView.SelectedItems[0].SubItems[1]);
        }

        private void editAcceptingLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageEdit(this.acceptingMetainstructionListView.SelectedItems[0].SubItems[1]);
        }

        private void addReducingMetainstructionButton_Click(object sender, EventArgs e)
        {
            ListViewItem reducingMetainstructionItem =
                this.reducingMetainstructionListView.Items.Add("Reducing Metainstruction");
            reducingMetainstructionItem.Checked = true;
            reducingMetainstructionItem.SubItems.Add("");
            reducingMetainstructionItem.SubItems.Add("");
            reducingMetainstructionItem.SubItems.Add("");
            reducingMetainstructionItem.SubItems.Add("");
            reducingMetainstructionItem.SubItems[0].Text =
                this.ListViewAddName(this.reducingMetainstructionListView,
                    "R", this.reducingNameTextBox.Text, 0);
            reducingMetainstructionItem.SubItems[1].Text = "null";
            reducingMetainstructionItem.SubItems[1].Tag = null;
            reducingMetainstructionItem.SubItems[4].Text = "null";
            reducingMetainstructionItem.SubItems[4].Tag = null;
        }

        private void deleteReducingMetainstructionButton_Click(object sender, EventArgs e)
        {
            this.reducingMetainstructionListView.Items.Remove(
                this.reducingMetainstructionListView.SelectedItems[0]);
        }

        private void wizardLeftLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageWizard(this.reducingMetainstructionListView.SelectedItems[0].SubItems[1]);
        }

        private void editLeftLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageEdit(this.reducingMetainstructionListView.SelectedItems[0].SubItems[1]);
        }

        private void wizardRightLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageWizard(this.reducingMetainstructionListView.SelectedItems[0].SubItems[4]);
        }

        private void editRightLanguageButton_Click(object sender, EventArgs e)
        {
            this.LanguageEdit(this.reducingMetainstructionListView.SelectedItems[0].SubItems[4]);
        }

        private void acceptingMetainstructionListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.UpdateAcceptingMetainstructionUI();
        }

        private void reducingMetainstructionListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.UpdateReducingMetainstructionUI();
        }

        private void fromWordTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                if (!this.alphabet.Contains(this.fromWordTextBox.Text))
                {
                    MessageBox.Show("All the letters of the from-word must be from the alphabet.", "Warning");
                    return;
                }
                this.reducingMetainstructionListView.SelectedItems[0].SubItems[2].Text =
                    this.fromWordTextBox.Text;
            }
        }

        private void fromWordTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.fromWordTextBox.Clear();
        }

        private void toWordTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                if (!this.alphabet.Contains(this.toWordTextBox.Text))
                {
                    MessageBox.Show("All the letters of the to-word must be from the alphabet.", "Warning");
                    return;
                }
                this.reducingMetainstructionListView.SelectedItems[0].SubItems[3].Text =
                    this.toWordTextBox.Text;
            }
        }

        private void toWordTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.toWordTextBox.Clear();
        }

        private void acceptingNameTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                this.acceptingMetainstructionListView.SelectedItems[0].SubItems[0].Text =
                    this.ListViewAddName(this.acceptingMetainstructionListView,
                        "A", this.acceptingNameTextBox.Text, 0);
            }
        }

        private void acceptingNameTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.acceptingNameTextBox.Clear();
        }

        private void reducingNameTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                this.reducingMetainstructionListView.SelectedItems[0].SubItems[0].Text =
                    this.ListViewAddName(this.reducingMetainstructionListView,
                        "R", this.reducingNameTextBox.Text, 0);
            }
        }

        private void reducingNameTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.reducingNameTextBox.Clear();
        }

        private void acceptAllCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckAll(this.acceptingMetainstructionListView);
        }

        private void acceptNullCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckNull(this.acceptingMetainstructionListView);
        }

        private void acceptInvCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckInv(this.acceptingMetainstructionListView);
        }

        private void reduceAllCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckAll(this.reducingMetainstructionListView);
        }

        private void reduceNullCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckNull(this.reducingMetainstructionListView);
        }

        private void reduceInvCheckButton_Click(object sender, EventArgs e)
        {
            this.ListViewCheckInv(this.reducingMetainstructionListView);
        }

        private void acceptingMetainstructionListView_DoubleClick(object sender, EventArgs e)
        {
            int selectedRow = this.acceptingMetainstructionListView.SelectedRow;
            int selectedCol = this.acceptingMetainstructionListView.SelectedCol;
            if (selectedRow != -1 && selectedCol != -1)
            {
                this.acceptingMetainstructionListView.Items[selectedRow].Checked =
                    !this.acceptingMetainstructionListView.Items[selectedRow].Checked;
                if (selectedCol == 0) this.acceptingNameTextBox.Focus();
                else if (selectedCol == 1)
                {
                    if (this.acceptingMetainstructionListView.
                            Items[selectedRow].SubItems[selectedCol].Tag != null)
                        this.LanguageEdit(this.acceptingMetainstructionListView.
                                Items[selectedRow].SubItems[selectedCol]);
                    else
                        this.LanguageWizard(this.acceptingMetainstructionListView.
                                Items[selectedRow].SubItems[selectedCol]);
                }
            }
        }

        private void reducingMetainstructionListView_DoubleClick(object sender, EventArgs e)
        {
            int selectedRow = this.reducingMetainstructionListView.SelectedRow;
            int selectedCol = this.reducingMetainstructionListView.SelectedCol;
            if (selectedRow != -1 && selectedCol != -1)
            {
                this.reducingMetainstructionListView.Items[selectedRow].Checked =
                    !this.reducingMetainstructionListView.Items[selectedRow].Checked;
                if (selectedCol == 0) this.reducingNameTextBox.Focus();
                else if (selectedCol == 1 || selectedCol == 4)
                {
                    if (this.reducingMetainstructionListView.
                            Items[selectedRow].SubItems[selectedCol].Tag != null)
                        this.LanguageEdit(this.reducingMetainstructionListView.
                                Items[selectedRow].SubItems[selectedCol]);
                    else
                        this.LanguageWizard(this.reducingMetainstructionListView.
                                Items[selectedRow].SubItems[selectedCol]);
                }
                else if (selectedCol == 2) this.fromWordTextBox.Focus();
                else if (selectedCol == 3) this.toWordTextBox.Focus();
            }
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.alphabet != null)
            {
                if (MessageBox.Show("If you change the alphabet all data will be lost.\r\n" +
                    "Do you really want to proceed?", "Confirmation", 
                    MessageBoxButtons.YesNo) == DialogResult.No)
                    return;
            }
            AlphabetForm alphabetForm = new AlphabetForm();
            DialogResult dialogResult = alphabetForm.ShowDialog();
            if (dialogResult == DialogResult.OK)
            {
                if (alphabetForm.Alphabet == "")
                {
                    this.alphabet = null;
                    this.restartingAutomaton = null;
                }
                else
                {
                    this.alphabet = new Alphabet<char>(alphabetForm.Alphabet);
                }
                this.ClearUI();
                this.UpdateUI();
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            DialogResult openFileDialogResult = openFileDialog.ShowDialog();
            if (openFileDialogResult == DialogResult.OK)
            {
                XmlSerializer restartingAutomatonSerializer =
                    new XmlSerializer(typeof(RestartingAutomatonXML<char>));
                FileStream fileLoadStream =
                    new FileStream(openFileDialog.FileName, FileMode.Open);
                RestartingAutomatonXML<char> restartingAutomatonXML =
                    restartingAutomatonSerializer.Deserialize(fileLoadStream)
                        as RestartingAutomatonXML<char>;
                fileLoadStream.Close();
                this.alphabet = new Alphabet<char>(restartingAutomatonXML);
                this.restartingAutomaton =
                    new RestartingAutomaton<char>(restartingAutomatonXML);
                this.DataToUI();
                this.UpdateUI();
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.UIToData(false, false, true))
            {
                SaveFileDialog saveFileDialog = new SaveFileDialog();
                saveFileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
                DialogResult saveFileDialogResult = saveFileDialog.ShowDialog();
                if (saveFileDialogResult == DialogResult.OK)
                {
                    XmlSerializer restartingAutomatonSerializer =
                        new XmlSerializer(typeof(RestartingAutomatonXML<char>));
                    FileStream fileSaveStream =
                        new FileStream(saveFileDialog.FileName, FileMode.Create);
                    RestartingAutomatonXML<char> restartingAutomatonXML =
                        new RestartingAutomatonXML<char>(this.restartingAutomaton);
                    restartingAutomatonSerializer.Serialize(fileSaveStream, restartingAutomatonXML);
                    fileSaveStream.Close();
                }
            }
        }

        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you really want to clear the restarting automaton?",
                "Confirmation", MessageBoxButtons.YesNo) == DialogResult.No)
                return;
            this.ClearUI();
            this.UpdateUI();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void wordToWordToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.UIToData(true, true, true))
            {
                WordToWordForm wordToWordForm =
                    new WordToWordForm(this.restartingAutomaton);
                wordToWordForm.ShowDialog();
            }
        }

        private void wordToAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.UIToData(true, true, true))
            {
                WordToAllForm wordToAllForm =
                    new WordToAllForm(this.restartingAutomaton);
                wordToAllForm.ShowDialog();
            }
        }

        private void startServerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.UIToData(true, true, true))
            {
                ServerForm serverForm = new ServerForm(this.restartingAutomaton);
                serverForm.ShowDialog();
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutForm aboutForm = new AboutForm();
            aboutForm.ShowDialog();
        }

        private void dFAModelerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form languageForm = new DFAModelerForm();
            languageForm.Show();
        }

        private void lstarAlgorithmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form languageForm = new LStarForm();
            languageForm.Show();
        }

        private void rpniAlgorithmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form languageForm = new RPNIForm();
            languageForm.Show();
        }

        private void regularExpressionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form languageForm = new RegexForm();
            languageForm.Show();
        }

        private void sltLanguageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form languageForm = new SLTForm();
            languageForm.Show();
        }

        private void RestartingAutomatonForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (e.CloseReason == CloseReason.UserClosing)
                e.Cancel = MessageBox.Show("Do you really want to close the RestartingAutomaton application?", 
                    "Confirmation", MessageBoxButtons.YesNo) == DialogResult.No;
        }
    }
}