using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.IO;

namespace RegularLanguages
{
    public partial class SLTPreviewForm : Form
    {
        public SLTPreviewForm(SLT<char> slt)
        {
            InitializeComponent();
            if (slt == null)
            {
                this.decideButton.Enabled = false;
            }
            else
            {
                this.slt = slt;
                this.sltPreviewTextBox.Text = slt.ToString();
                this.commentTextBox.Text = slt.Samples.Comment;
                this.decideButton.Enabled = true;
            }
        }

        private SLT<char> slt;

        public void Decide()
        {
            if (this.slt == null) return;
            string text = this.decideTextBox.Text;
            if (!this.slt.Alphabet.Contains(text))
            {
                MessageBox.Show("All the letters of the word must be from the alphabet.", "Warning");
                return;
            }
            if (this.slt.Decide(new GenericString<char>(text)))
                MessageBox.Show("Accept", "Information");
            else
                MessageBox.Show("Reject", "Information");
        }

        private void decideButton_Click(object sender, EventArgs e)
        {
            this.Decide();
            this.decideTextBox.Clear();
        }

        private void decideTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.Decide();
        }

        private void decideTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.decideTextBox.Clear();              
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*";
            DialogResult saveFileDialogResult = saveFileDialog.ShowDialog();
            if (saveFileDialogResult == DialogResult.OK)
            {
                XmlSerializer sltSerializer =
                    new XmlSerializer(typeof(SLTXML<char>));
                FileStream fileSaveStream =
                    new FileStream(saveFileDialog.FileName, FileMode.Create);
                SLTXML<char> sltXML =
                    new SLTXML<char>(this.slt);
                sltSerializer.Serialize(
                    fileSaveStream, sltXML);
                fileSaveStream.Close();
            }
        }

        private void copyButton_Click(object sender, EventArgs e)
        {
            XmlSerializer sltSerializer =
                new XmlSerializer(typeof(SLTXML<char>));
            StringWriter stringWriter = new StringWriter();
            SLTXML<char> sltXML = new SLTXML<char>(this.slt);
            sltSerializer.Serialize(stringWriter, sltXML);
            Clipboard.SetText(stringWriter.ToString());
            stringWriter.Close();
        }

        private void viewButton_Click(object sender, EventArgs e)
        {
            XmlSerializer sltSerializer =
                new XmlSerializer(typeof(SLTXML<char>));
            StringWriter stringWriter = new StringWriter();
            SLTXML<char> sltXML = new SLTXML<char>(this.slt);
            sltSerializer.Serialize(stringWriter, sltXML);
            XMLPreviewForm xmlPreviewForm = new XMLPreviewForm(stringWriter.ToString());
            stringWriter.Close();
            xmlPreviewForm.ShowDialog();
        }
    }
}