using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RegularLanguages;

namespace RestartingAutomaton
{
    public partial class ServerForm : Form
    {
        public ServerForm(RestartingAutomaton<char> restartingAutomaton)
        {
            InitializeComponent();
            this.restartingAutomaton = restartingAutomaton;
            this.StopWorkerUI();
        }

        private int channelPort;
        private string serviceName;

        private RestartingAutomaton<char> restartingAutomaton;

        private void StartWorkerUI()
        {
            this.channelPortTextBox.Enabled = false;
            this.serviceNameTextBox.Enabled = false;
            this.runButton.Enabled = false;
            this.stopButton.Enabled = true;
        }

        private void StopWorkerUI()
        {
            this.channelPortTextBox.Enabled = true;
            this.serviceNameTextBox.Enabled = true;
            this.runButton.Enabled = true;
            this.stopButton.Enabled = false;
        }

        private void runButton_Click(object sender, EventArgs e)
        {
            this.channelPort = this.channelPortTextBox.IntValue;
            this.serviceName = this.serviceNameTextBox.Text;
            if (channelPort < 8000 || channelPort > 8999)
            {
                MessageBox.Show("Please enter the port number between 8000 and 8999.", "Warning");
                return;
            }
            if (String.IsNullOrEmpty(serviceName))
            {
                MessageBox.Show("Please enter the nonempty service name.", "Warning");
                return;
            }
            if (this.serverBackgroundWorker.IsBusy)
            {
                MessageBox.Show("Background worker is running an asynchronous operation.", "Warning");
                return;
            }
            this.StartWorkerUI();
            this.serverBackgroundWorker.RunWorkerAsync();
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            this.serverBackgroundWorker.CancelAsync();
        }

        private void serverBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                RestartingAutomatonServer.Run(this.channelPort, this.serviceName,
                    this.restartingAutomaton, this.serverBackgroundWorker);
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Error");
            }
        }

        private void serverBackgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.StopWorkerUI();
        }
    }
}