using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RegularLanguages;

namespace RestartingAutomaton
{
    public partial class WordToAllForm : Form
    {
        public WordToAllForm()
        {
            InitializeComponent();
            this.restartingAutomaton = null;
            this.fromWord = null;
            this.UpdateUI();
        }

        public WordToAllForm(RestartingAutomaton<char> restartingAutomaton)
        {
            InitializeComponent();
            this.restartingAutomaton = restartingAutomaton;
            this.fromWord = null;
            this.UpdateUI();
        }

        private RestartingAutomaton<char> restartingAutomaton;
        private GenericString<char> fromWord;

        private void UpdateUI()
        {
            if (restartingAutomaton == null)
            {
                this.fromWordTextBox.Enabled = false;
                this.computeButton.Enabled = false;
            }
            else
            {
                this.fromWordTextBox.Enabled = true;
                this.computeButton.Enabled = true;
            }
            this.cancelButton.Enabled = false;
            this.okButton.Enabled = true;
        }

        private void StartWorkerUI()
        {
            this.fromWordTextBox.ReadOnly = true;
            this.computeButton.Enabled = false;
            this.cancelButton.Enabled = true;
            this.okButton.Enabled = false;
        }

        private void StopWorkerUI()
        {
            this.fromWordTextBox.ReadOnly = false;
            this.computeButton.Enabled = true;
            this.cancelButton.Enabled = false;
            this.okButton.Enabled = true;
        }

        private void computeButton_Click(object sender, EventArgs e)
        {
            this.fromWord = new GenericString<char>(this.fromWordTextBox.Text);
            if (!this.restartingAutomaton.Alphabet.Contains(this.fromWord))
            {
                MessageBox.Show("All the letters of the word must be from the alphabet.", "Warning");
                return;
            }
            if (this.wordToAllBackgroundWorker.IsBusy)
            {
                MessageBox.Show("Background worker is running an asynchronous operation.", "Warning");
                return;
            }
            this.StartWorkerUI();
            this.wordToAllBackgroundWorker.RunWorkerAsync();
        }

        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = this.restartingAutomaton.WordToAll(this.fromWord, this.wordToAllBackgroundWorker);
        }

        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.StopWorkerUI();
            if (e.Result is List<WordAcceptNames<char>>)
            {
                List<WordAcceptNames<char>> wordAcceptNamesList =
                    e.Result as List<WordAcceptNames<char>>;
                WordToAllFilterForm filterForm = new WordToAllFilterForm(
                    this.restartingAutomaton, this.fromWord, wordAcceptNamesList);
                filterForm.Show();
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.wordToAllBackgroundWorker.CancelAsync();
            this.StopWorkerUI();
        }
    }
}