using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Xml.Serialization;
using System.Windows.Forms;
using RegularLanguages;

namespace RestartingAutomaton
{
    public partial class WordToWordFilterForm : Form
    {
        public WordToWordFilterForm()
        {
            InitializeComponent();
            this.fromWord = null;
            this.toWord = null;
            this.wordToWordPathList = null;
            this.UpdateUI();
        }

        public WordToWordFilterForm(
            GenericString<char> fromWord,
            GenericString<char> toWord,
            List<WordToWordPath<char>> wordToWordPathList)
        {
            InitializeComponent();
            this.Text += " [" + fromWord.ToString() + " -> " + toWord.ToString() + "]";
            this.fromWord = fromWord;
            this.toWord = toWord;
            this.wordToWordPathList = wordToWordPathList;
            this.UpdateUI();
        }

        private GenericString<char> fromWord;
        private GenericString<char> toWord;
        private List<WordToWordPath<char>> wordToWordPathList;

        private bool UpdateUI()
        {
            if (this.wordToWordPathList != null)
            {
                this.filterTextBox.Enabled = true;
                this.filterButton.Enabled = true;
                this.filterRichTextBox.Enabled = true;
                Regex filterRegex = null;
                try
                {
                    filterRegex = new Regex(this.filterTextBox.Text);
                }
                catch
                {
                    MessageBox.Show("Invalid regex pattern.", "Warning");
                    return false;
                }
                this.filterRichTextBox.SuspendLayout();
                this.filterRichTextBox.Clear();
                foreach (WordToWordPath<char> wordPath in wordToWordPathList)
                {
                    string wordPathString = wordPath.ToString();
                    if (filterRegex.IsMatch(wordPathString))
                        wordPath.AppendToRichTextBox(this.filterRichTextBox);
                }
                this.filterRichTextBox.ResumeLayout(true);
                return true;
            }
            else
            {
                this.filterTextBox.Enabled = false;
                this.filterButton.Enabled = false;
                this.filterRichTextBox.Enabled = false;
                return false;
            }
        }

        private void filterTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
                this.UpdateUI();
        }

        private void filterButton_Click(object sender, EventArgs e)
        {
            this.UpdateUI();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}