using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RegularLanguages;

namespace RestartingAutomaton
{
    public partial class WordToWordForm : Form
    {
        public WordToWordForm()
        {
            InitializeComponent();
            this.restartingAutomaton = null;
            this.fromWord = null;
            this.toWord = null;
            this.UpdateUI();
        }

        public WordToWordForm(RestartingAutomaton<char> restartingAutomaton)
        {
            InitializeComponent();
            this.restartingAutomaton = restartingAutomaton;
            this.fromWord = null;
            this.toWord = null;
            this.UpdateUI();
        }

        private RestartingAutomaton<char> restartingAutomaton;
        private GenericString<char> fromWord;
        private GenericString<char> toWord;

        private void UpdateUI()
        {
            if (restartingAutomaton == null)
            {
                this.fromWordTextBox.Enabled = false;
                this.toWordTextBox.Enabled = false;
                this.computeButton.Enabled = false;
            }
            else
            {
                this.fromWordTextBox.Enabled = true;
                this.toWordTextBox.Enabled = true;
                this.computeButton.Enabled = true;
            }
            this.cancelButton.Enabled = false;
            this.okButton.Enabled = true;
        }

        private void StartWorkerUI()
        {
            this.fromWordTextBox.ReadOnly = true;
            this.toWordTextBox.ReadOnly = true;
            this.computeButton.Enabled = false;
            this.cancelButton.Enabled = true;
            this.okButton.Enabled = false;
        }

        private void StopWorkerUI()
        {
            this.fromWordTextBox.ReadOnly = false;
            this.toWordTextBox.ReadOnly = false;
            this.computeButton.Enabled = true;
            this.cancelButton.Enabled = false;
            this.okButton.Enabled = true;
        }

        private void computeButton_Click(object sender, EventArgs e)
        {
            this.fromWord = new GenericString<char>(this.fromWordTextBox.Text);
            this.toWord = new GenericString<char>(this.toWordTextBox.Text);
            if (!this.restartingAutomaton.Alphabet.Contains(this.fromWord) ||
                !this.restartingAutomaton.Alphabet.Contains(this.toWord))
            {
                MessageBox.Show("All the letters of the word must be from the alphabet.", "Warning");
                return;
            }
            if (this.wordToWordBackgroundWorker.IsBusy)
            {
                MessageBox.Show("Background worker is running an asynchronous operation.", "Warning");
                return;
            }
            this.StartWorkerUI();
            this.wordToWordBackgroundWorker.RunWorkerAsync();
        }

        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            e.Result = this.restartingAutomaton.WordToWordPaths(
                this.fromWord, this.toWord, this.wordToWordBackgroundWorker);
        }

        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.StopWorkerUI();
            if (e.Result is List<WordToWordPath<char>>)
            {
                List<WordToWordPath<char>> wordToWordPathList =
                    e.Result as List<WordToWordPath<char>>;
                WordToWordFilterForm filterForm = new WordToWordFilterForm(
                    this.fromWord, this.toWord, wordToWordPathList);
                filterForm.Show();
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.wordToWordBackgroundWorker.CancelAsync();
            this.StopWorkerUI();
        }
    }
}